@@ -73,7 +73,7 @@ def _array_to_pixmap(arr):
     if arr.dtype == np.float32 or arr.dtype == np.float64:
         arr = (255 * arr).round()
     arr = arr.astype(np.uint8)
-    return QtGui.QImage(arr.data, width, height, channel * width, QtGui.QImage.Format_RGB888)
+    return QtGui.QPixmap.fromImage(QtGui.QImage(arr.data, width, height, channel * width, QtGui.QImage.Format_RGB888))
 
 @functools.lru_cache(30)
 def _get_image(path):
@@ -720,10 +720,14 @@ class Label(QtWidgetComponent):
                 change_cursor = False
                 if self.props.selectable:
                     change_cursor = True
-                    interaction_flags |= (QtCore.Qt.TextSelectableByMouse | QtCore.Qt.TextSelectableByKeyboard)
+                    interaction_flags = (QtCore.Qt.TextSelectableByMouse | QtCore.Qt.TextSelectableByKeyboard)
                 if self.props.editable:
                     change_cursor = True
-                    interaction_flags |= QtCore.Qt.TextEditable
+                    # PyQt5 doesn't support bitwise or with ints
+                    if interaction_flags:
+                        interaction_flags |= QtCore.Qt.TextEditable
+                    else:
+                        interaction_flags = QtCore.Qt.TextEditable
                 if change_cursor and self.props.cursor is None:
                     commands.append((self.underlying.setCursor, _CURSORS["text"]))
                 if interaction_flags:
@@ -731,6 +735,35 @@ class Label(QtWidgetComponent):
         return commands
 
 
+class Image(QtWidgetComponent):
+    """An image container.
+
+    Args:
+        src: either the path to the image, or an np array. The np array must be 3 dimensional (height, width, channels)
+        scale_to_fit: if True, the image will be scaled to fit inside the container.
+    """
+
+    @register_props
+    def __init__(self, src: tp.Any = "", scale_to_fit: bool = True, **kwargs):
+        super().__init__(**kwargs)
+        self.underlying = None
+
+    def _initialize(self):
+        self.underlying = QtWidgets.QLabel()
+        self.underlying.setObjectName(str(id(self)))
+
+    def _qt_update_commands(self, children, newprops, newstate):
+        if self.underlying is None:
+            self._initialize()
+
+        commands = super()._qt_update_commands(children, newprops, newstate, self.underlying, None)
+        commands.append((self.underlying.setScaledContents, self.props.scale_to_fit))
+        for prop in newprops:
+            if prop == "src":
+                commands.append((self.underlying.setPixmap, _image_descriptor_to_pixmap(self.props.src)))
+        return commands
+
+
 class Completer(object):
 
     def __init__(self, options, mode="popup"):
@@ -897,6 +930,60 @@ class Dropdown(QtWidgetComponent):
         return commands
 
 
+class RadioButton(QtWidgetComponent):
+    """Radio buttons.
+
+    .. figure:: ../widget_images/radio_button.png
+       :width: 300
+
+       Three mutually exclusive radio buttons.
+
+    Radio buttons are used to specify a single choice out of many.
+    Radio buttons belonging to the same parent component are exclusive:
+    only one may be selected at a time.
+
+    Args:
+        checked: whether or not the checkbox is checked initially
+        text: text for the label of the checkbox
+        on_change: callback for when the check box state changes.
+            The callback receives the new state of the check box as an argument.
+    """
+
+    @register_props
+    def __init__(self, checked: bool = False, text: tp.Any = "",
+                 on_change: tp.Callable[[bool], None] = (lambda checked: None), **kwargs):
+        super().__init__(**kwargs)
+        self._connected = False
+        self.underlying = None
+
+    def _initialize(self):
+        self.underlying = QtWidgets.QRadioButton(str(self.props.text))
+        size = self.underlying.font().pointSize()
+        self._set_size(size * len(self.props.text), size)
+        self.underlying.setObjectName(str(id(self)))
+
+    def set_on_change(self, on_change):
+        def on_change_fun(checked):
+            return on_change(checked)
+        if self._connected:
+            self.underlying.toggled.disconnect()
+        self.underlying.toggled.connect(on_change_fun)
+        self._connected = True
+
+    def _qt_update_commands(self, children, newprops, newstate):
+        if self.underlying is None:
+            self._initialize()
+
+        commands = super()._qt_update_commands(children, newprops, newstate, self.underlying)
+        commands.append((self.underlying.setChecked, self.props.checked))
+        for prop in newprops:
+            if prop == "on_change":
+                commands.append((self.set_on_change, newprops[prop]))
+            elif prop == "text":
+                commands.append((self.underlying.setText, str(newprops[prop])))
+        return commands
+
+
 class CheckBox(QtWidgetComponent):
     """Checkbox widget.
 
@@ -1034,7 +1121,6 @@ class Slider(QtWidgetComponent):
             min_value, max_value = self.props.min_value, self.props.max_value
             value = int((value - min_value) / (max_value - min_value) * self._granularity)
 
-        commands.append((self.underlying.setValue, value))
         for prop in newprops:
             if prop == "on_change":
                 commands.append((self.set_on_change, newprops[prop]))
@@ -1047,6 +1133,7 @@ class Slider(QtWidgetComponent):
                     commands.extend([(self.underlying.setMinimum, self.props.min_value),
                                      (self.underlying.setMaximum, self.props.max_value),
                                     ])
+        commands.append((self.underlying.setValue, value))
         return commands
 
 
@@ -1061,14 +1148,6 @@ class _LinearView(QtWidgetComponent):
     def __del__(self):
         pass
 
-    def _delete_child(self, i):
-        child_node = self.underlying_layout.takeAt(i)
-        if child_node.widget():
-            child_node.widget().deleteLater() # setParent(self._garbage_collector)
-
-    def _soft_delete_child(self, i):
-        self.underlying_layout.takeAt(i)
-
     def _recompute_children(self, children):
         commands = []
         children = [child for child in children if child.component.underlying is not None]
@@ -1082,11 +1161,7 @@ class _LinearView(QtWidgetComponent):
 
         for i, old_child in reversed(list(enumerate(self._widget_children))):
             if old_child not in new_children:
-                if self.underlying_layout is not None:
-                    commands.append((self._delete_child, i))
-                    old_child._destroy_widgets()
-                else:
-                    commands.append((old_child.underlying.setParent, None))
+                commands.append((self._delete_child, i, old_child))
                 del self._widget_children[i]
 
         old_child_index = 0
@@ -1098,16 +1173,11 @@ class _LinearView(QtWidgetComponent):
 
             if old_child is None or child.component is not old_child:
                 if child.component not in self._already_rendered:
-                    if self.underlying_layout is not None:
-                        commands.append((self.underlying_layout.insertWidget, i, child.component.underlying))
-                    else:
-                        commands.append((child.component.underlying.setParent, self.underlying))
+                    commands.append((self._add_child, i, child.component.underlying))
                     old_child_index -= 1
                 else:
-                    if self.underlying_layout is not None:
-                        commands.extend([(self._soft_delete_child, i,), (self.underlying_layout.insertWidget, i, child.component.underlying)])
-                    else:
-                        commands.extend([(old_child.underlying.setParent, None), (child.component.underlying.setParent, self.underlying)])
+                    commands.extend([(self._soft_delete_child, i, old_child),
+                                     (self._add_child, i, child.component.underlying)])
 
             old_child_index += 1
             self._already_rendered[child.component] = True
@@ -1136,6 +1206,27 @@ class View(_LinearView):
         super().__init__(**kwargs)
         self.underlying = None
 
+    def _delete_child(self, i, old_child):
+        if self.underlying_layout is not None:
+            child_node = self.underlying_layout.takeAt(i)
+            if child_node.widget():
+                child_node.widget().deleteLater() # setParent(self._garbage_collector)
+        else:
+            old_child.underlying.setParent(None)
+        old_child._destroy_widgets()
+
+    def _soft_delete_child(self, i, old_child):
+        if self.underlying_layout is not None:
+            self.underlying_layout.takeAt(i)
+        else:
+            old_child.underlying.setParent(None)
+
+    def _add_child(self, i, child_component):
+        if self.underlying_layout is not None:
+            self.underlying_layout.insertWidget(i, child_component)
+        else:
+            child_component.setParent(self.underlying)
+
     def _initialize(self):
         self.underlying = QtWidgets.QWidget()
         layout = self.props.layout
@@ -1169,7 +1260,6 @@ class View(_LinearView):
         return commands
 
 
-
 class ScrollView(_LinearView):
     """Scrollable layout widget for grouping children together.
 
@@ -1194,6 +1284,18 @@ class ScrollView(_LinearView):
 
         self.underlying = None
 
+    def _delete_child(self, i, old_child):
+        child_node = self.underlying_layout.takeAt(i)
+        if child_node.widget():
+            child_node.widget().deleteLater() # setParent(self._garbage_collector)
+        old_child._destroy_widgets()
+
+    def _soft_delete_child(self, i, old_child):
+        self.underlying_layout.takeAt(i)
+
+    def _add_child(self, i, child_component):
+        self.underlying_layout.insertWidget(i, child_component)
+
     def _initialize(self):
         self.underlying = QtWidgets.QScrollArea()
         self.underlying.setWidgetResizable(True)
@@ -1217,6 +1319,46 @@ class ScrollView(_LinearView):
         return commands
 
 
+class TabView(_LinearView):
+    """Widget with multiple tabs.
+
+    .. figure:: ../widget_images/tab_view.png
+       :width: 300
+
+       A TabView with 2 children.
+
+    Args:
+        labels: The labels for the tabs. The number of labels must match the number of children.
+    """
+
+    @register_props
+    def __init__(self, labels=None, **kwargs):
+        super().__init__(**kwargs)
+        self.underlying = None
+
+    def _delete_child(self, i, old_child):
+        self.underlying.removeTab(i)
+
+    def _soft_delete_child(self, i, old_child):
+        self.underlying.removeTab(i)
+
+    def _add_child(self, i, child_component):
+        self.underlying.insertTab(i, child_component, self.props.labels[i])
+
+    def _initialize(self):
+        self.underlying = QtWidgets.QTabWidget()
+        self.underlying.setObjectName(str(id(self)))
+
+    def _qt_update_commands(self, children, newprops, newstate):
+        if len(children) != len(self.props.labels):
+            raise ValueError(f"The number of labels should be equal to the number of children for TabView {self}")
+        if self.underlying is None:
+            self._initialize()
+        commands = self._recompute_children(children)
+        commands.extend(super()._qt_update_commands(children, newprops, newstate, self.underlying, None))
+        return commands
+
+
 class CustomWidget(QtWidgetComponent):
     """Custom widgets that you can define.
 
